/* CHANGE DRIVE AND PATH SO IT FITS YOUR CHOICE OF DATABASE 

CREATE DATABASE 'C:\websted\2003\2i\hoteldatabase\hoteladm.gdb'
USER 'sysdba' PASSWORD 'masterkey';
*/

/* Domain-definitioner */
EXEC sp_addtype dom_HNo,		INTEGER;
EXEC sp_addtype dom_GNo,		INTEGER;
EXEC sp_addtype dom_RNo,		INTEGER;
EXEC sp_addtype dom_Name,	'VARCHAR(30)';
EXEC sp_addtype dom_Address,	'VARCHAR(50)';
EXEC sp_addtype dom_Type,    'CHAR(1)' 
EXEC sp_addtype dom_Price,	FLOAT 
EXEC sp_addtype dom_Date,	DATETIME;

/* Tabel-definitioner */
CREATE TABLE Hotel(
     Hotel_No  dom_HNo      NOT NULL,
     Name      dom_Name     NOT NULL,
     Address   dom_Address  NOT NULL,

PRIMARY KEY (Hotel_No)
);

CREATE TABLE Room(
     Room_No   dom_RNo    NOT NULL,
     Hotel_No  dom_HNo    NOT NULL,
     Types     dom_Type   DEFAULT 'S',
     Price     dom_Price,

CONSTRAINT checkType 
CHECK (Types IN ('D','F','S') OR Types IS NULL),
CONSTRAINT checkPrice 
CHECK (price BETWEEN 0 AND 9999),

FOREIGN KEY (Hotel_No) REFERENCES Hotel (Hotel_No)
ON UPDATE CASCADE ON DELETE NO ACTION,
PRIMARY KEY (Room_No, Hotel_No)
);

CREATE TABLE Guest (
     Guest_No  dom_GNo       NOT NULL,
     Name      dom_Name      NOT NULL,
     Address   dom_Address   NOT NULL,

PRIMARY KEY (Guest_No)
);

CREATE TABLE Booking(
     Hotel_No  dom_HNo   NOT NULL,
     Guest_No  dom_GNo   NOT NULL,
     Date_From dom_Date  NOT NULL,
     Date_To   dom_Date  NOT NULL,
     Room_No   dom_RNo   NOT NULL,
     CONSTRAINT Ukorrekt_periode
       CHECK ((Date_To > Date_From) AND (Date_From <= '01-jan-2008')),

FOREIGN KEY (Guest_No) REFERENCES Guest (Guest_No),
FOREIGN KEY (Room_No, Hotel_No) REFERENCES Room(Room_No, Hotel_No),
PRIMARY KEY (Hotel_No, Room_No, Date_From)
/* DET ANTAGES AT ET VAERELSE UDLEJES FOR MINDST EET DOEGN AF GANGEN
   DVS. PAA EN GIVEN "Date_From" KAN ET VRELSE KUN VAERE UDLEJET EEN GANG */
);


/* Trigger-definitioner til referentiel integritet */
SET TERM ^ ;

/* Foelgende 3 triggere er eksempler paa hvordan der kan etableres
   ON DELETE CASCADE for de 3 fremmednoegler. I alle tilfaelde boer
   dog vaere etableret ON DELETE NO ACTION hvorfor alle er sat INACTIVE */

/* ON DELETE CASCADE fra Guest til Booking */
CREATE TRIGGER Tri_Del_Guest FOR Guest
INACTIVE BEFORE DELETE
AS
BEGIN
  DELETE FROM Booking WHERE Guest.Guest_No = Booking.Guest_No;
END^

/* ON DELETE CASCADE fra Hotel til Room */
CREATE TRIGGER Tri_Del_Hotel FOR Hotel
INACTIVE BEFORE DELETE
AS
BEGIN
  DELETE FROM Room WHERE Hotel.Hotel_No = Room.Hotel_No;
END^

/* ON DELETE CASCADE fra Room til Booking */
CREATE TRIGGER Tri_Del_Room FOR Room
INACTIVE BEFORE DELETE
AS
BEGIN
  DELETE FROM Booking WHERE Booking.Room_No = Room.Room_No AND
                            Booking.Hotel_No = Room.Hotel_No;
END^

/* Foelgende 3 triggere er eksempler paa hvordan der kan etableres
   ON UPDATE CASCADE for de 3 fremmednoegler. */

/* ON UPDATE CASCADE fra Guest til Booking */
CREATE TRIGGER Tri_Upd_Guest FOR Guest
ACTIVE AFTER UPDATE
AS
BEGIN
  IF (NEW.Guest_No <> OLD.Guest_No) THEN
    UPDATE Booking SET Booking.Guest_NO = NEW.Guest_No
      WHERE Booking.Guest_No = OLD.Guest_No;
END^

/* ON UPDATE CASCADE fra Room til Booking */
CREATE TRIGGER Tri_Upd_Room FOR Room
ACTIVE AFTER UPDATE
AS
BEGIN
  IF (NEW.Room_No <> OLD.Room_No) THEN
    UPDATE Booking SET Booking.Room_No = NEW.Room_No
      WHERE Booking.Room_No = OLD.Room_No;
  IF (NEW.Hotel_NO <> OLD.Hotel_No) THEN
    UPDATE Booking SET Booking.Hotel_No = NEW.Hotel_NO
      WHERE Booking.Hotel_No = OLD.Hotel_No;
END^

/* ON UPDATE CASCADE fra Hotel til Booking */
CREATE TRIGGER Tri_Upd_Hotel FOR Hotel
ACTIVE AFTER UPDATE
AS
BEGIN
  IF (NEW.Hotel_No <> OLD.Hotel_No) THEN
    UPDATE Room SET Room.Hotel_No = NEW.Hotel_No
      WHERE Room.Hotel_No = OLD.Hotel_No;
END^

SET TERM ; ^

/* Check at et vaerelse ikke kan bookes hvis ikke det er er ledigt
   dvs. bookingperiode maa ikke overlappe eksisterende bookingperioder */
ALTER TABLE Booking
ADD
  CONSTRAINT Ikke_ledig
    CHECK (NOT EXISTS
      (SELECT *
        FROM Booking
        WHERE (
          ((Hotel_No = NEW.Hotel_No) AND (Room_No = NEW.Room_No))
          AND (   ((Date_From <  NEW.Date_From) AND (Date_To >  NEW.Date_From))
                   /* Booking omkranser NEW.Date_From */
               OR ((Date_From <  NEW.Date_To)   AND (Date_To >  NEW.Date_To))
                   /* Booking omkranser NEW.Date_To */
               OR ((Date_From >= NEW.Date_From) AND (Date_To <= NEW.Date_To))
                   /* Booking mellem NEW.Date_From og NEW.Date_To */
          ) /* AND */
        ) /* WHERE */
      ) /* SELECT */
    ) /* CHECK */
;

EXIT;
